/*

This software module was originally developed by 

Mike Guzewicz (E-mu Systems)


in the course of development of the MPEG-4 Audio (ISO/IEC 14496-3) standard. 
This software module is an implementation of a part of one or more 
MPEG-4 Audio (ISO/IEC 14496-3) tools as specified by the MPEG-4 Audio 
(ISO/IEC 14496-3). ISO/IEC gives users of the MPEG-4 Audio (ISO/IEC 14496-3) 
free license to this software module or modifications thereof for use 
in hardware or software products claiming conformance to the MPEG-4 Audio 
(ISO/IEC 14496-3). Those intending to use this software module in hardware 
or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for 
use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Audio (ISO/IEC 14496-3) conforming 
products. E-mu Systems retains full right to use the code for his/her own 
purpose, assign or donate the code to a third party and to inhibit third 
parties from using the code for non MPEG-4 Audio (ISO/IEC 14496-3) 
conforming products. This copyright notice must be included in all copies 
or derivative works. 

Copyright (C) 1997 E-mu Systems, Inc.

*/
     /*********************************************************************
     
     sfe_sflookup.h
     
     
     *********************************************************************/
#ifndef __SFLOOKUP_H
#define __SFLOOKUP_H

#include "sfe_datatype.h" 

#define MEMBEROFFSET(typ, mbr) \
	  ((WORD) ((CHAR*) &((typ*) 0)->mbr - (CHAR*) 0))

#ifdef _UNIX_STUB_
typedef DWORD SFLOOKUP;
#else
typedef WORD SFLOOKUP;
#endif

void InitLookup(void);
void ClearLookup(void);
const SFLOOKUP *GetLookup(void);

#endif
