/* $Id: saol_spatial.c,v 1.3 1997/11/20 16:40:30 eds Exp $ */
/* $Log: saol_spatial.c,v $
 * Revision 1.3  1997/11/20  16:40:30  eds
 * Added ISO header.
 *
 * Revision 1.3  1997/11/15 00:43:33  luked
 * This is the result of the manual merge from the vendor branch for the
 * release tagged Fribourg_after_integration. I (brian) have probably lost
 * some history, but oh well.
 *
 * This contains the integration work by Eric Scheirer (MIT).
 *
 * Revision 1.2  1997/09/17  14:11:26  eds
 * Created stub for future work.
 * */
/*********************************************************************

This software module was originally developed by

Eric D. Scheirer (MIT Media Laboratory)

in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard.  ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation.

This software module is hereby released into the public domain.

***********************************************************************/
#include "saol.h"
#include "saol_sched.h"
#include "saol_interp.h"


void spatialize(context *cx, double *pl, int p_ct) {
  /* pl[0] is the signal stream to spatialize;
     pl[1] is the azimuth;
     pl[2] is the elevation;
     pl[3] is the distance.

     cx->sa->all->g->outchan is the number of output channels
       (1-channel "spatialization" is easy!) */
  int i;
  
  /* .
     .
     . */

  for (i=0;i!=cx->sa->all->g->outchan;i++)
    cx->instr->output[i][cx->asample_ptr] += 0;  /* add output here */
}
