/* Copyright (C) 1997, ISO/IEC

The software module contained in this file was originally 
developed by Prof. Alexandros Eleftheriadis and Yihan Fang 
(Columbia University) as part of the Flavor run-time library 
(see http://www.ee.columbia.edu/flavor). Copyright has been 
transferred to ISO/IEC for inclusion in the MPEG-4 Reference 
Software (ISO/IEC 14496-5) distribution. ISO/IEC gives users 
of this code free license to this software or modifications 
thereof for use in hardware or software products claiming 
conformance to one or more parts of the MPEG-4 (ISO/IEC 14496) 
series of specifications. Those intending to use this Reference 
Software in hardware or software products are advised that 
its use may infringe existing patents. The original developer 
of this software module and his company, the subsequent editors 
and their companies, and ISO/IEC have no liability for use of 
this software or modifications thereof in an implementation. 
Copyright is not released for imlementations that do not comply 
to one or more parts of the MPEG-4 (ISO/IEC 14496) series of 
specifications. Columbia University retains full right to use 
the code for its own purpose, including assignment or donatation 
of this code to a third party.

This copyright notice must be included in all copies or derivative 
works.

For more information or to receive updated versions of this 
module, contact Prof. Alexandros Eleftheriadis at 
eleft@ee.columbia.edu.
*/
/*
 * $Id: port.h,v 1.2 1997/11/20 18:11:35 eds Exp eds $
 *
 * Copyright (c) 1996, 1997 ADVENT Project, Columbia University.
 * All rights reserved.
 *
 * This is unpublished proprietary software of the ADVENT Project at Columbia
 * University; it may not be disclosed to third parties, copied or duplicated
 * in any form, in whole or in part, without the prior written permission of
 * ADVENT. It may be shared with other departments within the same company;
 * however, ADVENT should be notified of such transfer in advance.
 * Furthermore, this software is distributed as is, with absolutely no warranty
 * of any kind, either expressed or implied, including, but not limited to, the
 * implied warranties of merchantability of fitness for a particular
 * purpose. The entire risk as to the quality and performance of the program(s)
 * is with you. Should the program(s) prove devective, you assume the cost of
 * all necessary servicing, repair or correction.  Appropriate acknowledgements
 * to the authors and ADVENT should be included in any publications or
 * demonstrations which are based on the direct use or modification of the
 * program(s). If you would like to use this software as part of a commercial
 * product, please contact ADVENT.
 *
 * Authors:
 *		Alexandros Eleftheriadis <eleft@ee.columbia.edu>
 */

#ifndef _port_h_
#define _port_h_

#ifdef _SunOS_
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#endif

#ifdef _Solaris_
#include <unistd.h>
extern char *strdup(const char *s);
#endif


#ifdef __sgi
#include <unistd.h>
#include <bstring.h>
#include <malloc.h>
#include <stdlib.h>
#define MAX_LEN 128
#endif

#ifdef __alpha
#define MAX_LEN 128
#include <stdlib.h>
#include <unistd.h>
#endif

#ifdef _AIX_
#include <stdlib.h>
#endif

#ifdef _HP_
#include <unistd.h>
#include <strings.h>
#endif

#if defined(_WIN32_) || defined(WIN32)
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>
#include <io.h>

/* max length for filenames */

#define MAX_LEN _MAX_PATH
#endif

#endif /* ! _port_h_ */
