/* $Id: saol_tok_table.h,v 1.1 1997/11/10 23:03:54 eds Exp $ */
/* $Log: saol_tok_table.h,v $
 * Revision 1.1  1997/11/10  23:03:54  eds
 * Initial revision
 * */

/*********************************************************************

This software module was originally developed by

Eric D. Scheirer (MIT Media Laboratory)

in the course of development of the MPEG-2 NBC/MPEG-4 Audio standard
ISO/IEC 13818-7, 14496-1,2 and 3. This software module is an
implementation of a part of one or more MPEG-2 NBC/MPEG-4 Audio tools
as specified by the MPEG-2 NBC/MPEG-4 Audio standard.  ISO/IEC gives
users of the MPEG-2 NBC/MPEG-4 Audio standards free license to this
software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-2 NBC/ MPEG-4 Audio
standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing
patents. The original developer of this software module and his/her
company, the subsequent editors and their companies, and ISO/IEC have
no liability for use of this software module or modifications thereof
in an implementation.

This software module is hereby released into the public domain.

***********************************************************************/
int is_builtin(char *);
int lexel_map(long);
char *tok_str(int);

#define TOK_AOPCODE 0
#define TOK_ASIG 1
#define TOK_ELSE 2
#define TOK_EXPORTS 3
#define TOK_EXTEND 4
#define TOK_GLOBAL 5
#define TOK_IF 6
#define TOK_IMPORTS 7
#define TOK_INCHANNELS 8
#define TOK_INPUTMOD 9
#define TOK_INSTR 10
#define TOK_IOPCODE 11
#define TOK_IVAR 12
#define TOK_KOPCODE 13
#define TOK_KRATE 14
#define TOK_KSIG 15
#define TOK_MAP 16
#define TOK_OPARRAY 17
#define TOK_OPCODE 18
#define TOK_OUTBUS 19
#define TOK_OUTCHANNELS 20
#define TOK_OUTPUT 21
#define TOK_RETURN 22
#define TOK_ROUTE 23
#define TOK_SEND 24
#define TOK_SEQUENCE 25
#define TOK_SBSYNTH 26
#define TOK_SPATIALIZE 27
#define TOK_SRATE 28
#define TOK_TABLE 29
#define TOK_TABLEMAP 30
#define TOK_TEMPLATE 31
#define TOK_TURNOFF 32
#define TOK_WHILE 33
#define TOK_WITH 34
#define TOK_XSIG 35
#define TOK_K_RATE 48
#define TOK_S_RATE 49
#define TOK_INCHAN 50
#define TOK_OUTCHAN 51
#define TOK_TIME 52
#define TOK_DUR 53
#define TOK_MIDICTRL 54
#define TOK_MIDITOUCH 55
#define TOK_MIDIBEND 56
#define TOK_INPUT 57
#define TOK_INGROUP 58
#define TOK_RELEASED 59
#define TOK_CPULOAD 60
#define TOK_POSITION 61
#define TOK_DIRECTION 62
#define TOK_LISTENERPOSITION 63
#define TOK_MINFRONT 64
#define TOK_MINBACK 65
#define TOK_MAXFRONT 66
#define TOK_MAXBACK 57
#define TOK_AND 80
#define TOK_OR 81
#define TOK_GEQ 82
#define TOK_LEQ 83
#define TOK_NEQ 84
#define TOK_EQEQ 85
#define TOK_MINUS 86
#define TOK_STAR 87
#define TOK_SLASH 88
#define TOK_PLUS 89
#define TOK_GT 90
#define TOK_LT 91
#define TOK_Q 92
#define TOK_COL 93
#define TOK_LP 94
#define TOK_RP 95
#define TOK_LC 96
#define TOK_RC 97
#define TOK_LB 98
#define TOK_RB 99
#define TOK_SEM 100
#define TOK_COM 101
#define TOK_EQ 102
#define TOK_NOT 103
#define TOK_SAMPLE 112
#define TOK_DATA 113
#define TOK_RANDOM 114
#define TOK_STEP 115
#define TOK_LINESEG 116
#define TOK_EXPSEG 117
#define TOK_CUBICSEG 118
#define TOK_POLYNOMIAL 119
#define TOK_WINDOW 120
#define TOK_HARM 121
#define TOK_HARM_PHASE 122
#define TOK_PERIODIC 123
#define TOK_BUZZ 124
#define TOK_CONCAT 125
#define TOK_EMPTY 126
#define TOK_CO_INT 128
#define TOK_FRAC 129
#define TOK_DBAMP 130
#define TOK_AMPDB 131
#define TOK_ABS 132
#define TOK_EXP 133
#define TOK_LOG 134
#define TOK_SQRT 135
#define TOK_SIN 136
#define TOK_COS 137
#define TOK_ATAN 138
#define TOK_POW 139
#define TOK_LOG10 140
#define TOK_ASIN 141
#define TOK_ACOS 142
#define TOK_FLOOR 143
#define TOK_CEIL 144
#define TOK_MIN 145
#define TOK_MAX 146
#define TOK_PCHOCT 147
#define TOK_OCTPCH 148
#define TOK_CPSPCH 149
#define TOK_PCHCPS 150
#define TOK_CPSOCT 151
#define TOK_OCTCPS 152
#define TOK_PCHMIDI 153
#define TOK_MIDIPCH 154
#define TOK_OCTMIDI 155
#define TOK_MIDIOCT 156
#define TOK_CPSMIDI 157
#define TOK_MIDICPS 158
#define TOK_FTLEN 160
#define TOK_FTLOOP 161
#define TOK_FTLOOPEND 162
#define TOK_FTSETLOOP 163
#define TOK_FTSETEND 164
#define TOK_FTBASECPS 165
#define TOK_FTSETBASE 166
#define TOK_TABLEREAD 167
#define TOK_TABLEWRITE 168
#define TOK_OSCIL 169
#define TOK_LOSCIL 170
#define TOK_DOSCIL 171
#define TOK_KOSCIL 172
#define TOK_KLINE 173
#define TOK_ALINE 174
#define TOK_KEXPON 176
#define TOK_AEXPON 177
#define TOK_KPHASOR 178
#define TOK_APHASOR 179
#define TOK_PLUCK 180
#define TOK_CO_BUZZ 181
#define TOK_FOF 182
#define TOK_IRAND 183
#define TOK_KRAND 184
#define TOK_ARAND 185
#define TOK_ILINRAND 186
#define TOK_KLINRAND 187
#define TOK_ALINRAND 188
#define TOK_IEXPRAND 189
#define TOK_KEXPRAND 190
#define TOK_AEXPRAND 191 
#define TOK_KPOISSONRAND 192
#define TOK_APOISSONRAND 193
#define TOK_IGAUSSRAND 194
#define TOK_KGAUSSRAND 195
#define TOK_AGAUSSRAND 196
#define TOK_PORT 197
#define TOK_HIPASS 198
#define TOK_LOPASS 199
#define TOK_BANDPASS 200
#define TOK_BANDSTOP 201
#define TOK_FIR 202
#define TOK_IIR 203
#define TOK_FIRT 204
#define TOK_IIRT 205
#define TOK_BIQUAD 206
#define TOK_FFT 207
#define TOK_IFFT 208
#define TOK_RMS 209
#define TOK_GAIN 210
#define TOK_BALANCE 211
#define TOK_DECIMATE 212
#define TOK_UPSAMP 213
#define TOK_DOWNSAMP 214
#define TOK_SAMPHOLD 215
#define TOK_DELAY 216
#define TOK_DELAY1 217
#define TOK_FDELAY 218
#define TOK_COMB 219
#define TOK_ALLPASS 220
#define TOK_CHORUS 221
#define TOK_FLANGE 222
#define TOK_REVERB 223
#define TOK_SYMBOL 240
#define TOK_INT 241
#define TOK_NUMBER 242
#define TOK_STRING 243
#define TOK_EOO 255
