# define IDENT 257
# define INTGR 258
# define NUMBER 259
# define STRCONST 260
# define AOPCODE 261
# define ELSE 262
# define EXPORTS 263
# define EXTEND 264
# define GLOBAL 265
# define IF 266
# define IMPORTS 267
# define INCHANNELS 268
# define INPUTMOD 269
# define INSTR 270
# define IOPCODE 271
# define IVAR 272
# define TABLE 273
# define KOPCODE 274
# define KRATE 275
# define KSIG 276
# define ASIG 277
# define MAP 278
# define OPARRAY 279
# define OPCODE 280
# define OUTBUS 281
# define OUTCHANNELS 282
# define OUTPUT 283
# define ROUTE 284
# define SEND 285
# define SEQUENCE 286
# define SRATE 287
# define TEMPLATE 288
# define TURNOFF 289
# define WHILE 290
# define WITH 291
# define XSIG 292
# define AND 293
# define OR 294
# define GEQ 295
# define LEQ 296
# define NEQ 297
# define EQEQ 298
# define MINUS 299
# define STAR 300
# define SPATIALIZE 301
# define SFSYNTH 302
# define TABLEMAP 303
# define SLASH 304
# define PLUS 305
# define GT 306
# define LT 307
# define Q 308
# define COL 309
# define LP 310
# define RP 311
# define LC 312
# define RC 313
# define LB 314
# define RB 315
# define SEM 316
# define COM 317
# define EQ 318
# define RETURN 319
# define NOT 320
# define ARRAYREF 321
# define OPCALL 322
# define IMPEXP 323
# define VARDECL 324
# define NOTAG 325
# define SPECIALOP 326
# define UNOT 327
# define UMINUS 328
# define HIGHEST 329
